using System;
using System.Windows.Forms;
using System.Collections;
using System.Reflection;

namespace MultiColumnTree
{
	/// <summary>
	/// The MCTNode class inherit the TreeNode class and hold the 
	/// connection to the source and the layout of the tree row.
	/// </summary>
	public class MCTNode : TreeNode {
		public MCTNode(){
		}
		public MCTNode(object source) {
			this.source = source;
		}
		/// <summary>
		/// Source class where the methodes are implemented to fill the columns
		/// </summary>
		private object source;
		public object Source {
			set {source = value;}
			get {return source;}
		}
		/// <summary>
		/// Layout of a tree row 
		/// </summary>
		private TreeRowLayout row;
		public TreeRowLayout Row {
			set {row = value;}
			get {return row;}
		}
		/// <summary>
		/// Flag that indicates the node as a folder.
		/// </summary>
		public bool IsFolder {
			get {return (bool)Invoker.GetProperty(source,"IsFolder");}
		}
		/// <summary>
		/// Get the last node filterd by check type
		/// </summary>
		/// <param name="filter">All, Checked, Unchecked</param>
		/// <returns>MCTNode</returns>
		new public MCTNode LastNode(Tree.FilterType filter) {
			MCTNode last = null;
			if (filter == Tree.FilterType.Checked) {
				for (int i = 0; i < Nodes.Count; i++) {
					if (Nodes[i].Checked) {
						last = (MCTNode)Nodes[i];
						//Console.WriteLine(((MCTNode)Nodes[i]).Row.FirstColumn.Text);
					}
				}
			} else if (filter == Tree.FilterType.Unchecked) {
				for (int i = 0; i < Nodes.Count; i++) {
					if (!Nodes[i].Checked) last = (MCTNode)Nodes[i];
				}
			} else {
				last = (MCTNode)Nodes[Nodes.Count-1];
			}
			return last;
		}	

		/// <summary>
		/// Override to get the right type
		/// </summary>
		new public MCTNode Parent {
			get {return (MCTNode)base.Parent;}
		}

	}
}
