using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace MultiColumnTree {
	/// <summary>
	/// MultiColumnTree build with PictureBoxes and Labels.
	/// </summary>
	/// 	
	public class Tree : System.Windows.Forms.UserControl, IMessageFilter {

		private System.Windows.Forms.ToolTip toolTip;
		private System.Windows.Forms.Panel treePanel;
		private System.Windows.Forms.Label lblheader;
		private System.Windows.Forms.Button button;
		private System.Windows.Forms.Label field;
		private System.Windows.Forms.TextBox textBox;
		private System.Windows.Forms.Panel headerPanel;
		private System.ComponentModel.IContainer components;

		#region Import
		[DllImport("user32.dll")] static extern uint GetSysColor(int nIndex);
		private const int COLOR_HIGHLIGHT = 13;
		private const int COLOR_HIGHLIGHTTEXT = 14;
		private const int COLOR_INACTIVE = 15;
		#endregion

		#region Con- De- structor
		public Tree() {
			// Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
			InitializeComponent();

			Application.AddMessageFilter(this);	
			this.rootNode = new MCTNode();
			this.filter = FilterType.All;
		}


		/// <summary> 
		/// Die verwendeten Ressourcen bereinigen.
		/// </summary>
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if(components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region IMessageFilter
		private const int WM_KEYDOWN = 0x0100;
		private const int VK_LEFT =  0x25;
		private const int VK_UP = 0x26;
		private const int VK_RIGHT = 0x27;
		private const int VK_DOWN = 0x28;
		private const int K_SPACE = 0x20;
		private const int WM_SCROLL = 0xa1;
		private const int WM_HSCROLL = 0x114;
		private const int WM_VSCROLL = 0x115;

		public bool PreFilterMessage(ref Message m) {
			if (selectedNodeControl == null) return false;

			if(!this.IsDisposed && button.Focused) {
				if(m.Msg == Convert.ToInt32(WM_KEYDOWN)) {
					if (m.WParam.ToInt32() == VK_LEFT) {
						Key_Left();
						return true;
					}
					if (m.WParam.ToInt32() == VK_RIGHT) {
						Key_Right();
						return true;
					}
					if (m.WParam.ToInt32() == VK_UP) {
						Key_Up();
						return true;
					}
					if (m.WParam.ToInt32() == VK_DOWN) {
						Key_Down();
						return true;
					}
					if (m.WParam.ToInt32() == K_SPACE) {
						Key_Space();
						return true;
					}
				}
			}
			return false;
		}
		#endregion

		#region Eventhandler
		/// <summary>
		/// Event after checked
		/// </summary>
		[Browsable(true)]
		public delegate void CheckChangedHandler(object sender, MultiColumnTree.EventArgs e);
		public event CheckChangedHandler CheckedChanged;
		/// <summary>
		/// Column change event handler type
		/// </summary>
		public delegate void ColumnChangedHandler(object sender, MultiColumnTree.EventArgs e);
		public event ColumnChangedHandler ColumnChanged;
		/// <summary>
		/// Event after checked
		/// </summary>
		[Browsable(true)]
		public delegate void SelectedHandler(object sender, MultiColumnTree.EventArgs e);
		public event SelectedHandler Selected;
		#endregion

		#region Vom Komponenten-Designer generierter Code
		/// <summary> 
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            this.field = new System.Windows.Forms.Label();
            this.toolTip = new System.Windows.Forms.ToolTip(this.components);
            this.treePanel = new System.Windows.Forms.Panel();
            this.textBox = new System.Windows.Forms.TextBox();
            this.lblheader = new System.Windows.Forms.Label();
            this.button = new System.Windows.Forms.Button();
            this.headerPanel = new System.Windows.Forms.Panel();
            this.treePanel.SuspendLayout();
            this.SuspendLayout();
            // 
            // field
            // 
            this.field.BackColor = System.Drawing.SystemColors.Window;
            this.field.Cursor = System.Windows.Forms.Cursors.Default;
            this.field.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
            this.field.Location = new System.Drawing.Point(344, 264);
            this.field.Name = "field";
            this.field.Size = new System.Drawing.Size(56, 16);
            this.field.TabIndex = 15;
            this.field.Text = "fieldFielField";
            this.field.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            this.field.Visible = false;
            // 
            // treePanel
            // 
            this.treePanel.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.treePanel.AutoScroll = true;
            this.treePanel.BackColor = System.Drawing.SystemColors.Window;
            this.treePanel.Controls.Add(this.textBox);
            this.treePanel.Location = new System.Drawing.Point(0, 22);
            this.treePanel.Name = "treePanel";
            this.treePanel.Size = new System.Drawing.Size(576, 456);
            this.treePanel.TabIndex = 0;
            // 
            // textBox
            // 
            this.textBox.Location = new System.Drawing.Point(320, 64);
            this.textBox.MaxLength = 16;
            this.textBox.Name = "textBox";
            this.textBox.Size = new System.Drawing.Size(100, 20);
            this.textBox.TabIndex = 0;
            this.textBox.Visible = false;
            this.textBox.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.textBox_KeyPress);
            this.textBox.Leave += new System.EventHandler(this.textBox_Leave);
            // 
            // lblheader
            // 
            this.lblheader.BackColor = System.Drawing.SystemColors.GradientActiveCaption;
            this.lblheader.Cursor = System.Windows.Forms.Cursors.Default;
            this.lblheader.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.lblheader.Location = new System.Drawing.Point(352, 296);
            this.lblheader.Name = "lblheader";
            this.lblheader.Size = new System.Drawing.Size(56, 23);
            this.lblheader.TabIndex = 17;
            this.lblheader.Text = "Header";
            this.lblheader.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            this.lblheader.Visible = false;
            // 
            // button
            // 
            this.button.Location = new System.Drawing.Point(536, 40);
            this.button.Name = "button";
            this.button.Size = new System.Drawing.Size(24, 23);
            this.button.TabIndex = 18;
            this.button.GotFocus += new System.EventHandler(this.button_GotFocus);
            this.button.Leave += new System.EventHandler(this.button_Leave);
            // 
            // headerPanel
            // 
            this.headerPanel.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.headerPanel.AutoScroll = true;
            this.headerPanel.BackColor = System.Drawing.SystemColors.Window;
            this.headerPanel.Location = new System.Drawing.Point(0, 0);
            this.headerPanel.Name = "headerPanel";
            this.headerPanel.Size = new System.Drawing.Size(560, 56);
            this.headerPanel.TabIndex = 20;
            // 
            // Tree
            // 
            this.BackColor = System.Drawing.SystemColors.Control;
            this.Controls.Add(this.field);
            this.Controls.Add(this.lblheader);
            this.Controls.Add(this.treePanel);
            this.Controls.Add(this.headerPanel);
            this.Controls.Add(this.button);
            this.Name = "Tree";
            this.Size = new System.Drawing.Size(576, 480);
            this.Load += new System.EventHandler(this.MultiColumnTree_Load);
            this.treePanel.ResumeLayout(false);
            this.treePanel.PerformLayout();
            this.ResumeLayout(false);

		}
		#endregion

		#region Property private + public
		private bool noTextChangedEvent = false;
		private int rowIndex;
		private int colIndex;
		private ArrayList rows;
		private ArrayList hLines;
		private Point[] vLines;

		private bool[] indentLine = new bool[100];
		private int indent;
		private Color backColor;
		private Color textColor;
		private Color backColorSelected;
		private Color textColorSelected;
		private Color textColorInactive;
		private const int ROWHEIGHT = 21;
		private Panel panel;
		private Control newControl;
		private Control oldControl;

		private Control selectedNodeControl;
		private MCTNode selectedNode;


		public enum FilterType {
			All, Checked, Unchecked
		}

		private FilterType filter;
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public FilterType Filter {
			set {filter = value;}
			get {return filter;}
		}
		[Browsable(false)]
		public object SelectedObject {
			get {
				if (selectedNodeControl.Tag == null) return null;
				else return ((MCTNode)selectedNodeControl.Tag).Source;
			}
		}
		private MCTNode rootNode = null;
		[DefaultValue(typeof(MCTNode))]
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public MCTNode RootNode {
			set {rootNode = value;}
			get {return rootNode;}
		}
		private string[] columnBindings = new string[]{};
		[Category("Columns")]
		[Browsable(true)]
		public string[] Bindings {
			set {columnBindings = value;}
			get {return columnBindings;}
		}
		private string[] captions = new string[]{};
		[Category("Columns")]
		[Browsable(true)]
		public string[] Captions {
			set {
				captions = value;}
			get {return captions;}
		}
		private ContextMenu[] folderContextMenues = new ContextMenu[]{};
		[Browsable(true)]
		[Category("Columns")]
		public ContextMenu[] FolderContextMenues {
			set {folderContextMenues = value;}
			get {return folderContextMenues;}
		}
		private ContextMenu[] fileContextMenues = new ContextMenu[]{};
		[Browsable(true)]
		[Category("Columns")]
		public ContextMenu[] FileContextMenues {
			set {fileContextMenues = value;}
			get {return fileContextMenues;}
		}
		private bool withCheckBoxes = true;
		[Browsable(true)]
		[DefaultValue(true)]
		[Category("Columns")]
		public bool WithCheckBoxes {
			set {withCheckBoxes = value;}
			get {return withCheckBoxes;}
		}
		#endregion

		#region Create Controls
		private void CreateTree(MCTNode node) {
			foreach (MCTNode tn in node.Nodes) {
				TreeRowLayout row = CreateTreeRow(tn);
				if ((bool)Invoker.GetProperty(tn.Source,"Hidden") == false) {
					rows.Add(row);
				}
				indentLine[indent] = IsLastNode(node.Nodes, tn);
				indent++;
				if (tn.IsExpanded) CreateTree(tn);
				indent--;
			}
		}
		private TreeRowLayout CreateHeaderRow(MCTNode treeNode) {
			TreeRowLayout row;
			if (treeNode.Row == null) {
				colIndex = 0;
				row = new TreeRowLayout(indent);
				row.AddColumn(CreateHeaderLabel("0")); 
				for (int i = 1; i < columnBindings.Length; i++) {
					row.AddColumn(CreateHeaderLabel(i.ToString()));
				}
				treeNode.Row = row;
			} else {
				row = treeNode.Row;
			}
			rowIndex++;
			return row;
		}

		private TreeRowLayout CreateTreeRow(MCTNode treeNode) {
			TreeRowLayout row;
			if (treeNode.Row == null) {
				colIndex = 0;
				row = new TreeRowLayout(indent);
				CreateIndent(row, treeNode);
				CreateTreeSymbol(row, treeNode);
				if (WithCheckBoxes)	CreateCheckedSymbol(row, treeNode);
				row.AddColumn(CreateControl(treeNode));
				for (int i = 1; i < columnBindings.Length; i++) {
					row.AddColumn(CreateControl(treeNode));
				}
				treeNode.Row = row;
			} else {
				row = treeNode.Row;
				CreateTreeSymbol(row, treeNode);
			}
			rowIndex++;
			return row;
		}

		private TreeLabel CreateHeaderLabel (string text) {
			TreeLabel lbl = new TreeLabel();
			lbl.BorderStyle = lblheader.BorderStyle;
			lbl.BackColor = lblheader.BackColor;
			lbl.TextAlign = lblheader.TextAlign;
			lbl.Font = lblheader.Font;
			lbl.Cursor = lblheader.Cursor;
			lbl.Location = new Point(0,0);
			lbl.Name = rowIndex.ToString() + ";" + colIndex.ToString();
			lbl.Height = ROWHEIGHT;
			if (captions.Length > 0) {
				lbl.Text = captions[colIndex];
				colIndex++;
			}
			return lbl;
		}
		private TreeLabel CreateControl (MCTNode treeNode) {
			TreeLabel ctrl = new TreeLabel();
			ctrl.BorderStyle = this.field.BorderStyle;
			ctrl.TextAlign = this.field.TextAlign;
			ctrl.AutoSize = false;
			ctrl.Cursor = this.field.Cursor;
			ctrl.TabStop = false;
			ctrl.TabIndex = 10; //rowIndex;
			ctrl.Location = new Point(0,0);
			ctrl.Name = rowIndex.ToString() + ";" + colIndex.ToString();
			ctrl.Height = ROWHEIGHT;
			ctrl.Width = 0;
			ctrl.Tag = treeNode;
			if (Bindings.Length > 0) {
				ctrl.PropertyName = Bindings[colIndex];
				if (treeNode.IsFolder) {
					if (colIndex < FolderContextMenues.Length) ctrl.ContextMenu = FolderContextMenues[colIndex];
				} else {
					if (colIndex < FileContextMenues.Length) ctrl.ContextMenu = FileContextMenues[colIndex];
				}
			}
			//this.toolTip.SetToolTip(lbl, lbl.Text);
			ctrl.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ctrl_MouseDown);
			ctrl.TextChanged += new System.EventHandler(this.ctrl_TextChanged);
			colIndex++;
			return ctrl;
		}
		private void CreateIndent(TreeRowLayout row, MCTNode treeNode) {
			for (int i = 0; i < indent; i++) {
				if (indentLine[i]) {
					row.AddIndent(new TreeSymbol(treeNode,ROWHEIGHT,false));
				} else {
					row.AddIndent(new TreeSymbol(treeNode,ROWHEIGHT,true));
				}
			}
		}

		private void CreateTreeSymbol(TreeRowLayout row, MCTNode treeNode) {
			row.TreePicture = new TreeSymbol(treeNode, ROWHEIGHT, filter); // treeSymbol;
			row.TreePicture.Click += new System.EventHandler(this.pbxTree_Click);
		}

		private void CreateCheckedSymbol(TreeRowLayout row, MCTNode treeNode) {
			Check checkPicture = new Check();
			checkPicture.CheckedChanged += new MultiColumnTree.Tree.CheckChangedHandler(checkPicture_CheckedChanged);
			row.CheckPicture = checkPicture;
			checkPicture.Tag = treeNode;
			// read from roperty checked in source object
			checkPicture.Checked = (bool) Invoker.GetProperty (((MCTNode)treeNode).Source, "Checked");
		}

		public void OpenTextBox() {
			newControl = selectedNodeControl;
			if (textBox.Visible) button.Focus(); // for leave event
			textBox.Location = newControl.Location;
            textBox.Top = newControl.Location.Y + newControl.Parent.Location.Y;
            textBox.Size = newControl.Size;
			//textBox.Width = 120;
			textBox.TabIndex = 0;
			textBox.Name = "Textbox";
			textBox.Visible = true;
			textBox.Focus();
			textBox.Text = newControl.Text;
			oldControl = newControl;
		}

		public void CloseTextBox() {
			if (textBox.Visible) {
				textBox.Visible = false;
				if (oldControl.Text != textBox.Text) {
					oldControl.Text = textBox.Text;
					if (ColumnChanged != null) {
						MultiColumnTree.EventArgs args = new EventArgs();
						ColumnChanged(oldControl, args);
					}
					ShowTree();
				}
				textBox.Text = "";
			}
		}
		public void Caption (int column, string text) {
			captions[column] = text;
			(rows[0] as TreeRowLayout).Controls[column].Text = text;
		}

		private void CreatePanel(Size size) {
			treePanel.SuspendLayout();
			treePanel.AutoScrollPosition = new Point(0, 0);

			panel = new Panel();
			panel.SuspendLayout();
			panel.AutoScroll = false;
			panel.Name = "panel";
			panel.Location = new System.Drawing.Point(0, 0);
			panel.Size = size;
			panel.Paint += new System.Windows.Forms.PaintEventHandler(this.panel_Paint);
			treePanel.Controls.Clear();
			treePanel.Controls.Add(textBox);
			treePanel.Controls.Add(panel);
			panel.ResumeLayout();
			treePanel.ResumeLayout();

			headerPanel.Controls.Clear();
		}
		#endregion

		#region Methodes
		public void ShowTree() {
			if (this.rootNode == null) return;
			noTextChangedEvent = true;
			Cursor cur = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;
			rows = TreeRowLayout.Rows;// new ArrayList();
			rows.Clear();
			rowIndex = 0;
			indent = 0;
			rows.Add(CreateHeaderRow(rootNode));

			CreateTree(rootNode);

			UpdateTree();
			Size size = CalculateLayout();
			CreatePanel(size);

			this.SuspendLayout();
			treePanel.SuspendLayout();
			headerPanel.SuspendLayout();
			panel.SuspendLayout();
			bool first = true;
			int bottom = 0;
			int right = 0;
			foreach (TreeRowLayout row in rows) {
				if (first) {
					row.AddControls(headerPanel.Controls);
					first = false;
				} else {
					row.AddControls(panel.Controls);
				}
				bottom = row.FirstColumn.Bottom;
				right = row.LastColumn.Right;
			}
			panel.ResumeLayout();
			headerPanel.ResumeLayout();
			treePanel.ResumeLayout();
			this.ResumeLayout();
			noTextChangedEvent = false;
			if (selectedNode != null)
				SelectControl(selectedNode.Row.FirstColumn);
			else
				if (rows.Count > 1)
					SelectControl((rows[1] as TreeRowLayout).FirstColumn);
			Cursor.Current = cur;
			int x = 1;
			if (right+1 > treePanel.Width) x += 16;
			if (bottom+x < treePanel.Height) {
				headerPanel.Width = treePanel.Width;
			} else {
				headerPanel.Width = treePanel.Width - 16;
			}
		}

		private Size CalculateLayout() {
			// calculate max width for the columns
			int[] maxWidth = new int[Math.Max(columnBindings.Length,1)];
			for (int col = 0; col < maxWidth.Length; col++) {
				maxWidth[col] = TreeRowLayout.MaxColumnWidth(col);
			}
			hLines = new ArrayList();
			vLines = new Point[Math.Max(columnBindings.Length,1)];
			int y = 0; // the y position for horizontal lines
			int yInc = -1;
			foreach (TreeRowLayout row in rows) {
				row.SetColumnWidth(maxWidth);
				row.SetLocations(y);
				y += yInc;
				hLines.Add(new Point(row.HLineX(), y++));
				yInc = ROWHEIGHT;
			}
			int x = 0; // the x position for vertical lines
			for (int i = 0; i < maxWidth.Length; i++) {
				x += maxWidth[i];
				//vLines[i] = new Point(x++, ROWHEIGHT); 
				vLines[i] = new Point(x++, 0); 
			}
			treePanel.Top = ROWHEIGHT + 1;
			treePanel.Height = this.Height - treePanel.Top;
			return new Size(x, y);
		}

		private bool IsLastNode (TreeNodeCollection nodes, MCTNode treeNode) {
			MCTNode last = null;
			if (filter == FilterType.Checked) {
				for (int i = 0; i < nodes.Count; i++) {
					if (nodes[i].Checked) {
						last = (MCTNode)nodes[i];
						//Console.WriteLine(((MCTNode)nodes[i]).Row.FirstColumn.Text + "----" + treeNode.Row.FirstColumn.Text);
					}
				}
			} else if (filter == FilterType.Unchecked) {
				for (int i = 0; i < nodes.Count; i++) {
					if (!nodes[i].Checked) last = (MCTNode)nodes[i];
				}
			} else {
				last = (MCTNode)nodes[nodes.Count-1];
			}
			return (last == treeNode);
		}

		private Color UIntToColor (uint val) {
			int r = (int)(val & 0xFF);
			int g = (int)((val >> 8) & 0xFF);
			int b = (int)((val >> 16) & 0xFF);
			Color color = Color.FromArgb(r,g,b);
			return color;
		}

		public void UpdateTree () {
			this.SuspendLayout();
			foreach (TreeRowLayout row in rows) { // visible rows
				if (row.CheckPicture != null) {
					object obj = (row.CheckPicture.Tag as MCTNode).Source;
					bool check = (bool) Invoker.GetProperty(obj,"Checked");
					if (row.CheckPicture.Checked != check) row.CheckPicture.Checked = check;
					((MCTNode)row.CheckPicture.Tag).Checked = check;
				}
				int i = 0;
				foreach (Control ctrl in row.Columns) {
					if (ctrl.Tag != null) {
						object obj = (ctrl.Tag as MCTNode).Source;
						if (columnBindings.Length > 0){
							string text = (string) Invoker.GetProperty(obj,columnBindings[i++]);
							if (text == null) text = "";
							if (ctrl.Text != text) ctrl.Text = text;
						}
					}
				}
			}
			if (panel != null) panel.Size = CalculateLayout();
			this.ResumeLayout();
		}
		private void SyncPanel() {
			if (headerPanel.AutoScrollPosition.X != treePanel.AutoScrollPosition.X)
				headerPanel.AutoScrollPosition = new Point(Math.Abs(treePanel.AutoScrollPosition.X), 0);
		}
		#endregion

		#region Key events
		private void Key_Down () {
			Control ctrl = TreeRowLayout.NextVisibleMCTNode(selectedNodeControl);
			SelectControl(ctrl);
			treePanel.ScrollControlIntoView(ctrl);
		}
		private void Key_Up () {
			Control ctrl = TreeRowLayout.PrevVisibleMCTNode(selectedNodeControl);
			SelectControl(ctrl);
			treePanel.ScrollControlIntoView(ctrl);
		}
		private void Key_Left () {
			Control ctrl = (Control)((MCTNode)selectedNodeControl.Tag).Row.PrevColumn(selectedNodeControl);
			SelectControl(ctrl);
			Control first = (Control)((MCTNode)selectedNodeControl.Tag).Row.FirstColumn;
			if (ctrl == first) 
				treePanel.ScrollControlIntoView( (Control)((MCTNode)ctrl.Tag).Row.FirstColumn );
			else 
				treePanel.ScrollControlIntoView(ctrl);
		}
		private void Key_Right () {
			Control ctrl = (Control)((MCTNode)selectedNodeControl.Tag).Row.NextColumn(selectedNodeControl);
			SelectControl(ctrl);
			treePanel.ScrollControlIntoView(ctrl);
		}
		private void Key_Space () {
			MCTNode treeNode = (MCTNode)selectedNodeControl.Tag;
			Check ctrl = (Check)((MCTNode)selectedNodeControl.Tag).Row.CheckPicture;
			ctrl.Checked = ! ctrl.Checked;
		}
		#endregion

		#region Events
		private void MultiColumnTree_Load(object sender, System.EventArgs e) {
			backColor = this.field.BackColor;
			textColor = this.field.ForeColor;
			backColorSelected = UIntToColor(GetSysColor(COLOR_HIGHLIGHT));
			textColorSelected = UIntToColor(GetSysColor(COLOR_HIGHLIGHTTEXT));
			textColorInactive = UIntToColor(GetSysColor(COLOR_INACTIVE));
		}

		/// <summary>
		/// Draw the grid
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void panel_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			if (hLines == null) return;
			if (vLines == null) return;
			Pen pen = new Pen(Color.LightGray, 1);
			foreach (Point p in hLines) {
				e.Graphics.DrawLine(pen, p.X, p.Y, (sender as Control).Width, p.Y);
			}
			foreach (Point p in vLines) {
				e.Graphics.DrawLine(pen, p.X, p.Y, p.X, (sender as Control).Height);
			}
			SyncPanel();
			//Console.WriteLine("paint");
		}
		/// <summary>
		/// Select a cell
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void ctrl_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
			SelectControl(sender as Control);
		}
	
		/// <summary>
		/// Click on tree symbol expand or collapse the subtree
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pbxTree_Click(object sender, System.EventArgs e) {
			button.Focus(); // for leave event
			ShowTree();
			TreeSymbol ctrl = (TreeSymbol)sender;
			MCTNode node =(MCTNode)ctrl.Tag;
			SelectControl(node.Row.FirstColumn);
		}

		/// <summary>
		/// Set select colors
		/// </summary>
		/// <param name="ctrl"></param>
		private void SelectControl(Control ctrl) {
			button.Focus();
			if (selectedNodeControl != null) {
				selectedNodeControl.BackColor = backColor;
				selectedNodeControl.ForeColor = textColor;
			}
			if (ctrl != null) {
				ctrl.BackColor = backColorSelected;
				ctrl.ForeColor = textColorSelected;
			}
			selectedNodeControl = ctrl;
			selectedNode = ctrl.Tag as MCTNode;
			if (Selected != null) 
				Selected(ctrl, new MultiColumnTree.EventArgs()); // raise selected event
		}

		private void button_GotFocus(object sender, System.EventArgs e) {
			if (rootNode == null) return;
			if (rootNode.Nodes.Count == 0) return;
//			if (nodes[0].Tag == null) return;
			if(selectedNodeControl == null) 
				SelectControl( ((MCTNode)rootNode.Nodes[0]).Row.FirstColumn );
			else
				SelectControl(selectedNodeControl);
		}

		private void button_Leave(object sender, System.EventArgs e) {
			if (selectedNodeControl != null) {
				selectedNodeControl.BackColor = textColorInactive;
				selectedNodeControl.ForeColor = textColor;
			}
		}

		private void ctrl_TextChanged(object sender, System.EventArgs es) {
			if (noTextChangedEvent) return;
			TreeLabel ctrl = (TreeLabel)sender;
			object source =((MCTNode)ctrl.Tag).Source;
			string propertyName = ctrl.PropertyName; //columnBindings[Convert.ToInt32(ctrl.Name.Substring(ctrl.Name.LastIndexOf(";")+1))];
			Invoker.SetProperty(source, propertyName, new object[]{((Control)sender).Text});
			if (ColumnChanged != null) {
				MultiColumnTree.EventArgs args = new EventArgs();
				ColumnChanged(ctrl, args);
			}
		}
		private void checkPicture_CheckedChanged(object sender, MultiColumnTree.EventArgs e) {
			SelectControl(selectedNodeControl);	
			MultiColumnTree.Check check = (sender as MultiColumnTree.Check);
			MCTNode node = check.Tag as MCTNode;
			node.Checked = e.BoolValue;
			if (CheckedChanged != null) CheckedChanged(node, e);

			Control ctrl = (Control)((MCTNode)(sender as Check).Tag).Row.FirstColumn;
			if (ctrl != null) SelectControl(ctrl);
		}

		private void textBox_Leave(object sender, System.EventArgs e) {
			CloseTextBox();
		}

		private void textBox_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e) {
			if (e.KeyChar == '\r') {
				button.Focus(); // for leave event
				e.Handled = true;
			} else if (e.KeyChar == (char)Keys.Escape) {
				textBox.Text = oldControl.Text; // cancel save the old value
				button.Focus(); // for leave event
				e.Handled = true;
			}
		}
		#endregion

	}
	/// <summary>
	/// Class to hold the event arguments
	/// </summary>
	public class EventArgs : System.EventArgs {
		public EventArgs () {
			BoolValue = false;
		}
		public EventArgs (bool val) {
			BoolValue = val;
		}
		public bool BoolValue;
		public bool Cancel = false;
	}

}
