using System;
using System.Collections;
using System.Windows.Forms;
using System.Drawing;

namespace MultiColumnTree
{
	/// <summary>
	/// TreeRowLayout contains all information and controls to view a tree row.
	/// </summary>
	public class TreeRowLayout
	{

		public TreeRowLayout(int indent)
		{
			this.indent = indent;
			this.columns = new ArrayList();
			this.indentPictures = new ArrayList();
		}

		#region Static
		static public ArrayList Rows = new ArrayList();

		static public Control NextVisibleMCTNode (Control ctrl) {
			int idx = Rows.IndexOf((object)  ((MCTNode)ctrl.Tag).Row  );
			if (idx < Rows.Count - 1) {
				TreeRowLayout row = (TreeRowLayout)Rows[idx + 1];
				return row.FirstColumn;
			}
			return ctrl;
		}
		static public Control PrevVisibleMCTNode (Control ctrl) {
			int idx = Rows.IndexOf((object)  ((MCTNode)ctrl.Tag).Row  );
			if (idx > 1) {
				TreeRowLayout row = (TreeRowLayout)Rows[idx - 1];
				return row.FirstColumn;
			}
			return ctrl;
		}
		static public int MaxColumnWidth (int index) {
			int max = 0;
			foreach (TreeRowLayout row in Rows) {
				int width = row.ColumnWidth(index);
				if (width > max) max = width;
			}
			return max;
		}

		#endregion

		#region Property
		private ArrayList columns;
		public ArrayList Columns {
			set {columns = value;}
			get {return columns;}
		}
		public Control[] ColumnList {
			get {return (Control[])columns.ToArray(typeof(Control));}
		}
		private ArrayList controls;
		public Control[] Controls {
			get {
				if (controls == null) {
					controls = new ArrayList();
					if (indentPictures.Count > 0) controls.AddRange(indentPictures);
					if (treePicture != null) controls.Add(treePicture);
					if (checkPicture != null) controls.Add(checkPicture);
					controls.AddRange(columns);
				}
				return (Control[])controls.ToArray(typeof(Control));
			}
		}
		private int indent;
		public int Indent {
			set {indent = value;}
			get {return indent;}
		}
		private TreeSymbol treePicture;
		public TreeSymbol TreePicture {
			set {treePicture = value;}
			get {return treePicture;}
		}
		private Check checkPicture;
		public Check CheckPicture {
			set {checkPicture = value;}
			get {return checkPicture;}
		}
		private ArrayList indentPictures;
		public ArrayList IndentPictures {
			set {indentPictures = value;}
			get {return indentPictures;}
		}
		#endregion

		#region Methodes
		public void AddColumn (TreeLabel ctrl) {
			columns.Add(ctrl);
		}
		public void AddIndent (Control ctrl) {
			indentPictures.Add(ctrl);
		}
		public Control FirstColumn {
			get {return (Control)columns[0];}
		}
		public Control LastColumn {
			get {return (Control)columns[columns.Count-1];}
		}
		public int ColumnWidth (int idx) {
			TreeLabel ctrl = (TreeLabel)columns[idx];
			if (idx == 0) return ctrl.WidthMin + GraphicWidth();
			else return ctrl.WidthMin;
		}
		public Control NextColumn (Control ctrl) {
			int idx = columns.IndexOf(ctrl);
			if (idx < columns.Count-1) return (Control)columns[idx + 1];
			return ctrl;
		}
		public Control PrevColumn (Control ctrl) {
			int first = columns.IndexOf(this.FirstColumn);
			int idx = columns.IndexOf(ctrl);
			if (idx > first) return (Control)columns[idx - 1];
			return ctrl;
		}
		public void SetColumnWidth(int[] maxWidth) {
			int idx = 0;
			foreach (Control ctrl in columns) {
				if (idx == 0) ctrl.Width = maxWidth[idx++] - GraphicWidth();
				else ctrl.Width = maxWidth[idx++];
			}
		}
		public int GraphicWidth () {
			int x = 0;
			foreach (Control ctrl in indentPictures) {
				x += ctrl.Width;
			}
			if (treePicture != null)x += treePicture.Width;
			if (checkPicture != null) x += checkPicture.Width;
			return x;
		}
		public void SetLocations (int y) {
			int x = 0;
			foreach (Control ctrl in indentPictures) {
				ctrl.Location = new Point(x ,y);
				x += ctrl.Width;
			}
			if (treePicture != null) {
				treePicture.Location = new Point(x ,y);
				x += treePicture.Width;
			}
			if (checkPicture != null) {
				checkPicture.Location = new Point(x ,y);
				x += checkPicture.Width;
				//Console.WriteLine(checkPicture.Location);
			}
			foreach (Control ctrl in columns) {
				ctrl.Location = new Point(x ,y);
				x += ctrl.Width + 1;
			}
		}
		public int HLineX () {
			if (checkPicture != null) return checkPicture.Right;
			else return 0;
		}

		public void AddControls (Control.ControlCollection controls) {
			Control ctrl;
			for (int i = 0; i < indentPictures.Count; i++) {
				ctrl = (Control)indentPictures[i];
				controls.Add(ctrl);
			}
			if (treePicture != null) {
				controls.Add(treePicture);
			}
			if (checkPicture != null) {
				controls.Add(checkPicture);
			}
			for (int i = 0; i < columns.Count; i++) {
				controls.Add((Control)columns[i]);
			}
		}
		#endregion
	}
}
